var/sd_window/MOTD/MOTD = new()

mob/Stat()
	if(client)
		stat("Enhanced windows:", "[client.sd_enhanced_windows?"En":"Dis"]abled")
		stat("OS:",client.sd_OS)
		stat("browser:",client.sd_browser_name)
		stat("browser version:",client.sd_browser_version)
		stat("browser minor version:",client.sd_browser_minor_version)

		if(client.sd_windows)
			stat("Viewing [client.sd_windows.len] sd_windows")
			for(var/I in client.sd_windows)
				stat("\ref[I]")
		else
			stat("Viewing no sd_windows")

client
	var/tmp/sd_window/S		// Stores a persistent dialog that the user can interact with

	New()
		..()
		world << "<b>[src] logs in.</b>"
		src << "<b>All of the text output shown in this text area is generated by the demo, not \
			the library.</b>\nThe core library never outputs anything but the dialog boxes you \
			instruct it to."

		new/sd_window/detect_navigator(src)	// check for enhanced mode

		spawn(5)	// allow a few ticks for OS detection
			if(sd_OS == "Win32") sd_enhanced_windows = 1
			MOTD.Display(src)	//display the MOTD window


	Del()
		world << "<b>[src] logs out.</b>"
		del(mob)
		..()

	verb
		Background()
			set desc = "Display a window with a background image."

			// y larger than screen height results in y = screen_height-height
			var/sd_window/W = new(x=0, y=5000, height=200)
			// changing content after the window is created so it can contain \ref to itself
			W.content = {"<head><title>Background Image</title>
				<style>
					body {background: #FFEEEE url("demobg.png");}
				</style>
				</head><body>This window has a background image, so I had to use
					/sd_window.AddResource() to make sure the library sends the image to every
					client who views the window.
				</body>"}
			W.AddResource('bg.dmi', "demobg.png")
			W.Display(src)

		Close_All()
			set desc = "Close all your open windows"

			for(var/sd_window/W in sd_windows)
				W.Close(src)

		Dialog_Close()
			set desc = "Close the interactive dialog window, but preserve its data."

			world << "[src] is closing \ref[S] without the remove flag, so that it is not \
				deleted and may be reopened with current values."
			S.Close(src,,0)

		Dialog_Delete()
			set desc = "Delete the interactive dialog window. If open, it will close automatically."
			del(S)

		Dialog_Open()
			set desc = "Open an interactive dialog window."

			if(!S)
				S = new(x=5000, y=0, width=500, height=244)
				// changing content after the window is created so it can contain \ref to itself
				S.DemoContent()
			S.Display(src)

		Enhanced()
			set desc = "Toggle wether sd_window displays enhanced windows to your client."
			sd_enhanced_windows = !sd_enhanced_windows
			if(sd_enhanced_windows)
				src << "Enabled enhanced windows."
			else
				src << "Disabled enhanced windows."

		Limits()
			set desc = "Display a demo window describing some limits of sd_windows."

			// x larger than screen width results in x = screen_width-width
			var/sd_window/W = new(x=5000, y=237, height=350)
			// changing content after the window is created so it can contain \ref to itself
			W.content = {"<head><title>sd_window Limits &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; \
				&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; \
				&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; \
				&nbsp; &nbsp;</title><script>
				function BYOND(X) {
					var wndw = window;
					if(top==self) wndw = dialogArguments;
					wndw.document.location.href = 'BYOND://?src=\ref[W]&'+X;
				}
				</script>
				<style>
				body {background: #FFEEEE;}
				.pseudo {color:blue;text-decoration:underline;cursor:pointer}
				</style>
				</head><body>
				The biggest limitation of sd_windows is a problem with links, but there is a
				workaround:<br>
				<a href=http://shadowdarke.byond.com/>External link</a> - This normal link
					opens a new window.<br>
				<a href=BYOND://?src=\ref[W]&Clicked_BYOND_Link>BYOND Link</a> - Normal BYOND
					links like this one don't work.<br>
				<span onClick='BYOND("Clicked_BYOND_Pseudolink");' class=pseudo>Pseudo-link
					</span> - This fake link passes info back to BYOND as you would expect a
					normal BYOND link to.
				<p>I also haven't figured out a way to get rid of the titlebar yet, but you can
					hide the "-- Web Page Dialog" part by adding a long string of " &amp;nbsp;"
					to the document title like this window does."}
			W.Display(src)

		Random(N as num)
			set desc = "(number) Open a number of random sd_windows. Too many windows may cause \
				problems."

			for(var/loop = 1 to N)
				var/sd_window/W = new()
				W.x = rand(-1,724)
				W.y = rand(-1,743)
				W.height=25
				W.content = "Random window at ([W.x], [W.y])"
				W.Display(src)
				sleep(1)

		Say(T as text)
			set desc = "(message) Broadcast a message to other users."

			world << "<b>[src]:</b> [copytext(html_encode(T),1,512)]"

		Simple()
			set desc = "This simple window only requires one line of code to create and display."

			new /sd_window("This is a simple window invoked with only one line of code:<P>\
				<i>new /sd_window(\"This is a...\", src)</i>\
				<p>It isn't even stored (except internal to the library), just fire and forget. \
				The library will still let us know when it has been closed.", src)

		Timed()
			set desc = "Open a timed window that closes in 45 seconds."

			new /sd_window("<head><title>Timed Dialog</title></head><body bgcolor=#EEFFFF>This \
				timed window will close automatically in 45 seconds.</body>", src, 450, x=0, \
				height=100)

sd_window
	Closed(client/M)
		world << "[M] closed [src] (\ref[src])."
		..()
	NoViewers()
		world << "No clients are looking at [src] (\ref[src]) anymore."
		..()
	Topic(href,hlist[])
		world << "[src] (\ref[src]) reports from [usr]:"
		for(var/index in hlist - "src")
			world << "\t[index][hlist[index]?" = [hlist[index]]":""]"
		if(hlist["chng"])
			usr.client.S.vars[hlist["chng"]] = text2num(hlist["val"])
			usr.client.S.Close(usr,0,0)
			usr.client.S.DemoContent()
			usr.client.S.Display(usr)
		else if(hlist["chck"])
			var/chng
			switch(hlist["chck"])
				if("SDW_FOCUS") chng = SDW_FOCUS
				if("SDW_RESIZE") chng = SDW_RESIZE
				if("SDW_STATUS") chng = SDW_STATUS
				if("SDW_SCROLL") chng = SDW_SCROLL
				if("SDW_SUNKEN") chng = SDW_SUNKEN
				if("SDW_HELP") chng = SDW_HELP
			if(hlist["val"]=="true")
				usr.client.S.flags |= chng
			else
				usr.client.S.flags &= ~chng
			usr.client.S.Close(usr,0,0)
			usr.client.S.DemoContent()
			usr.client.S.Display(usr)
		..()
	proc/DemoContent()
		content = {"<head><title>Demo Dialog</title><script>
function BYOND(X) {
	var wndw = window;
	if(top==self) wndw = dialogArguments;
	wndw.document.location.href = 'BYOND://?src=\ref[src]&'+X;
}
function chng(X) {BYOND("chng=" + X.name + "&val=" + X.value)}
function chck(X) {BYOND("chck=" + X.name + "&val=" + X.checked)}
window.onhelp=function(){alert("You can use the javascript onHelp event to do something nifty \
	when people use the context-sensitive help feature (the question mark (?) icon that appears \
	next to the dialog's close button when you use the SDW_HELP flag.)")}
</script>
<style>
body {background: #EEEEFF;}
</style>
</head><body>
<P>You can report anything from this dialog back to BYOND with a little javascript. The \
	examples below update this dialog when you change them.<br>
Position: <INPUT NAME='x' VALUE='[x]' SIZE=1 onChange='chng(this);'> x \
	<INPUT NAME='y' VALUE='[y]' SIZE=1 onChange='chng(this);'> (negative values center the \
		dialog)<br>
Size: <INPUT NAME='width' VALUE='[width]' SIZE=1 onChange='chng(this);'> x \
	<INPUT NAME='height' VALUE='[height]' SIZE=1 onChange='chng(this);'><br>
<INPUT NAME='SDW_FOCUS' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_FOCUS?" CHECKED":""]> SDW_FOCUS<BR>
<INPUT NAME='SDW_RESIZE' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_RESIZE?" CHECKED":""]> SDW_RESIZE<BR>
<INPUT NAME='SDW_STATUS' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_STATUS?" CHECKED":""]> SDW_STATUS<BR>
<INPUT NAME='SDW_SCROLL' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_SCROLL?" CHECKED":""]> SDW_SCROLL<BR>
<INPUT NAME='SDW_SUNKEN' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_SUNKEN?" CHECKED":""]> SDW_SUNKEN<BR>
<INPUT NAME='SDW_HELP' TYPE=CHECKBOX onClick='chck(this);'\
	[flags&SDW_HELP?" CHECKED":""]> SDW_HELP
<INPUT TYPE=SUBMIT style="position:absolute;left:-300px;">
</body>"}
		/*	SDW_FOCUS		If not set, window restores focus to Dream Seeker
			SDW_RESIZE		Window may be resized
			SDW_STATUS		Display the status bar
			SDW_SCROLL		Display scroll bars
			SDW_PRINT		Hide from print and print preview
			SDW_SUNKEN		Show a sunken window edge
			SDW_HELP		Display context sensitive help icon
			SDW_UNADORNED	Disable window border chrome */

/* demonstration events
window.onhelp = function(){BYOND('F1 pressed')}
window.onfocus = function(){BYOND('Dialog recieved focus')}
window.onmove = function(){BYOND('Dialog moved')}
window.onresize = function(){BYOND('Dialog resized')}
window.onscroll = function(){BYOND('Dialog scrolled')}
*/

	MOTD
		// a sample Message Of The Day display
		content = {"<head><title>Demo Message of the Day</title><style>
body {background: #EEFFEE;}
</style>
</head><body>This is a demo MOTD display that opens when you connect to the world. It uses a
	single sd_window object that multiple clients can view.
<p>Some demo windows will open as soon as you close this window.
<p align=center><input type=button value='Close this window' onClick='window.close();'></p>
</body>"}

		Closed(client/M)
			world << "[M] closed [src]. The other windows will open now."
			M.Dialog_Open()
			spawn(2) M.Limits()
			spawn(4) M.Background()
			spawn(6) M.Timed()
			spawn(8) M.Simple()

		NoViewers()
			// override so that the MOTD doesn't delete itself when no one is looking at it
			world << "No one is viewing [src] (\ref[src]) anymore, but it won't automatically \
				delete because I overrode the NoViewers() proc. That way the program doesn't \
				have to keep making MOTD objects each time someone connects."